from dataclasses import dataclass
from typing import Dict, List, Optional

from .channel import Channel
from .cost import Cost
from .criterion_requirement import CriterionRequirement
from .evidence import Evidence
from .harvest_meta_data import HarvestMetaData
from .legal_resource import LegalResource
from .output import Output
from .participation import Participation
from .publisher import Publisher
from .rule import Rule
from .schema_contact_point import SchemaContactPoint
from .skos_code import SkosCode
from .skos_concept import SkosConcept


@dataclass
class PublicService:
    id: Optional[str] = None
    uri: Optional[str] = None
    identifier: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    isDescribedAt: Optional[List[SkosConcept]] = None
    isGroupedBy: Optional[List[str]] = None
    hasCompetentAuthority: Optional[List[Publisher]] = None
    harvest: Optional[HarvestMetaData] = None
    keyword: Optional[List[Dict[str, str]]] = None
    sector: Optional[List[SkosConcept]] = None
    isClassifiedBy: Optional[List[SkosConcept]] = None
    language: Optional[List[SkosCode]] = None
    hasCriterion: Optional[List[CriterionRequirement]] = None
    hasParticipation: Optional[List[Participation]] = None
    hasInput: Optional[List[Evidence]] = None
    produces: Optional[List[Output]] = None
    requires: Optional[List["PublicService"]] = None
    hasContactPoint: Optional[List[SchemaContactPoint]] = None
    follows: Optional[List[Rule]] = None
    hasLegalResource: Optional[List[LegalResource]] = None
    hasChannel: Optional[List[Channel]] = None
    processingTime: Optional[str] = None
    hasCost: Optional[List[Cost]] = None
    relation: Optional[List["PublicService"]] = None
    spatial: Optional[List[str]] = None
    associatedBroaderTypesByEvents: Optional[List[str]] = None
    type: str = "publicservices"



import logging
from typing import List, Optional

from rdflib import BNode, Graph, URIRef

from fdk_rdf_parser.classes import MediaTypeOrExtent
from fdk_rdf_parser.rdf_utils import dcat_uri, dct_uri


def extract_fdk_format(
    graph: Graph, subject: URIRef
) -> Optional[List[MediaTypeOrExtent]]:
    fdk_format: List[MediaTypeOrExtent] = list()
    dct_format = extract_media_type_list(graph, subject, dct_uri("format"))
    fdk_format.extend(dct_format if dct_format else [])
    dcat_media_type = extract_media_type_list(graph, subject, dcat_uri("mediaType"))
    fdk_format.extend(dcat_media_type if dcat_media_type else [])

    return fdk_format if len(fdk_format) > 0 else None


def extract_media_type(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[MediaTypeOrExtent]:
    value = graph.value(subject, predicate)
    if not value:
        return None
    elif isinstance(value, BNode):
        logging.error(
            f"Unable to parse {predicate.toPython()} for {subject.toPython()}"
        )
        return None
    elif isinstance(value, URIRef):
        return MediaTypeOrExtent(uri=value.toPython())
    else:
        return MediaTypeOrExtent(code=value.toPython())


def extract_media_type_list(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[MediaTypeOrExtent]]:
    media_types = []
    for obj in graph.objects(subject, predicate):
        if isinstance(obj, BNode):
            logging.error(
                f"Unable to parse {predicate.toPython()} for {subject.toPython()}"
            )
        elif isinstance(obj, URIRef):
            media_types.append(MediaTypeOrExtent(uri=obj.toPython()))
        else:
            media_types.append(MediaTypeOrExtent(code=obj.toPython()))
    return media_types if len(media_types) > 0 else None



from dataclasses import dataclass
from typing import Dict, Optional

from fdk_rdf_parser.classes import (
    EuDataTheme,
    LosNode,
    MediaTypeOrExtent,
    MediaTypeOrExtentType,
    SkosCode,
)
from .reference_data_client import get_new_reference_data, get_reference_data
from .utils import remove_scheme_and_trailing_slash


@dataclass
class DataServiceReferenceData:
    media_types: Optional[Dict[str, MediaTypeOrExtent]] = None


@dataclass
class DatasetReferenceData:
    provenancestatement: Optional[Dict[str, SkosCode]] = None
    rightsstatement: Optional[Dict[str, SkosCode]] = None
    frequency: Optional[Dict[str, SkosCode]] = None
    linguisticsystem: Optional[Dict[str, SkosCode]] = None
    referencetypes: Optional[Dict[str, SkosCode]] = None
    openlicenses: Optional[Dict[str, SkosCode]] = None
    location: Optional[Dict[str, SkosCode]] = None
    eu_data_themes: Optional[Dict[str, EuDataTheme]] = None
    los_themes: Optional[Dict[str, LosNode]] = None
    media_types: Optional[Dict[str, MediaTypeOrExtent]] = None


@dataclass
class InformationModelReferenceData:
    linguisticsystem: Optional[Dict[str, SkosCode]] = None
    openlicenses: Optional[Dict[str, SkosCode]] = None
    location: Optional[Dict[str, SkosCode]] = None
    eu_data_themes: Optional[Dict[str, EuDataTheme]] = None
    los_themes: Optional[Dict[str, LosNode]] = None


@dataclass
class PublicServiceReferenceData:
    linguisticsystem: Optional[Dict[str, SkosCode]] = None


def get_data_service_reference_data() -> DataServiceReferenceData:
    return DataServiceReferenceData(media_types=get_and_map_media_types())


def get_dataset_reference_data() -> DatasetReferenceData:
    return DatasetReferenceData(
        provenancestatement=get_and_map_reference_codes("provenancestatement"),
        rightsstatement=get_and_map_reference_codes("rightsstatement"),
        frequency=get_and_map_reference_codes("frequency"),
        linguisticsystem=get_and_map_reference_codes("linguisticsystem"),
        referencetypes=get_and_map_reference_codes("referencetypes"),
        openlicenses=get_and_map_reference_codes("openlicenses"),
        location=get_and_map_reference_codes("location"),
        eu_data_themes=get_and_map_eu_data_themes(),
        los_themes=get_and_map_los_themes(),
        media_types=get_and_map_media_types(),
    )


def get_info_model_reference_data() -> InformationModelReferenceData:
    return InformationModelReferenceData(
        linguisticsystem=get_and_map_reference_codes("linguisticsystem"),
        openlicenses=get_and_map_reference_codes("openlicenses"),
        location=get_and_map_reference_codes("location"),
        eu_data_themes=get_and_map_eu_data_themes(),
        los_themes=get_and_map_los_themes(),
    )


def get_public_service_reference_data() -> PublicServiceReferenceData:
    return PublicServiceReferenceData(
        linguisticsystem=get_and_map_reference_codes("linguisticsystem")
    )


def get_and_map_reference_codes(endpoint: str) -> Optional[Dict[str, SkosCode]]:
    codes = get_reference_data(f"/codes/{endpoint}")
    if codes is not None:
        return {
            remove_scheme_and_trailing_slash(str(code.get("uri"))): SkosCode(
                uri=str(code.get("uri")) if code.get("uri") is not None else None,
                code=str(code.get("code")) if code.get("code") is not None else None,
                prefLabel=code.get("prefLabel")
                if code.get("prefLabel") is not None
                else None,
            )
            for code in codes
        }
    else:
        return None


def get_and_map_eu_data_themes() -> Optional[Dict[str, EuDataTheme]]:
    mapped = {}
    themes = get_new_reference_data("/eu/data-themes").get("dataThemes")
    if themes is not None:
        for theme in themes:
            eu_theme = EuDataTheme()
            eu_theme.add_values_from_dict(theme)
            mapped[remove_scheme_and_trailing_slash(str(theme.get("uri")))] = eu_theme
    return mapped if len(mapped) > 0 else None


def get_and_map_los_themes() -> Optional[Dict[str, LosNode]]:
    mapped = {}
    los_nodes = get_new_reference_data("/los/themes-and-words").get("losNodes")
    if los_nodes is not None:
        for los_node in los_nodes:
            los_theme = LosNode()
            los_theme.add_values_from_dict(los_node)
            mapped[
                remove_scheme_and_trailing_slash(str(los_node.get("uri")))
            ] = los_theme
    return mapped if len(mapped) > 0 else None


def get_and_map_media_types() -> Optional[Dict[str, MediaTypeOrExtent]]:
    media_types = {}
    iana_codes = get_new_reference_data("/iana/media-types").get("mediaTypes")
    if iana_codes is not None:
        for code in iana_codes:
            media_type_uri = str(code["uri"]) if code.get("uri") else None
            if media_type_uri:
                media_types[
                    remove_scheme_and_trailing_slash(media_type_uri)
                ] = MediaTypeOrExtent(
                    uri=code.get("uri"),
                    name=f"{code.get('name')}",
                    code=f"{code.get('type')}/{code.get('subType')}",
                    type=MediaTypeOrExtentType.MEDIA_TYPE,
                )

    file_types = get_new_reference_data("/eu/file-types").get("fileTypes")
    if file_types is not None:
        for file_type in file_types:
            file_type_uri = str(file_type["uri"]) if file_type.get("uri") else None
            if file_type_uri:
                media_types[
                    remove_scheme_and_trailing_slash(file_type_uri)
                ] = MediaTypeOrExtent(
                    uri=file_type.get("uri"),
                    name=f"{file_type.get('code')}",
                    code=f"{file_type.get('code')}",
                    type=MediaTypeOrExtentType.FILE_TYPE,
                )

    return media_types if len(media_types) > 0 else None



from dataclasses import dataclass
from typing import Dict, Optional

from .skos_code import SkosCode


@dataclass
class Format:
    uri: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    format: Optional[str] = None
    language: Optional[SkosCode] = None



import os
from typing import Dict, List, Optional

from fdk_rdf_parser.classes import EuDataTheme, LosNode, Reference, SkosCode

base_url = os.getenv(
    "REFERENCE_DATA_BASE_URI",
    "http://staging.fellesdatakatalog.digdir.no/reference-data",
)

new_base_url = os.getenv(
    "NEW_REFERENCE_DATA_BASE_URI",
    "https://www.staging.fellesdatakatalog.digdir.no/new-reference-data",
)


def new_reference_data_url(endpoint: str) -> str:
    return f"{new_base_url}{endpoint}"


def reference_data_url(endpoint: str) -> str:
    return f"{base_url}{endpoint}"


def extend_reference_types(
    references: Optional[List[Reference]],
    reference_types: Optional[Dict[str, SkosCode]],
) -> Optional[List[Reference]]:
    if references is None or reference_types is None:
        return references
    else:
        extended_references = []
        for ref in references:
            ref.referenceType = extend_skos_code(ref.referenceType, reference_types)
            extended_references.append(ref)
        return extended_references


def split_themes(
    themes: Optional[List[EuDataTheme]],
    los: Optional[Dict[str, LosNode]],
) -> Dict[str, List[EuDataTheme]]:
    splitted_themes: Dict[str, List[EuDataTheme]] = {"los": [], "eu_data_themes": []}
    if themes is None:
        return splitted_themes
    elif los is None:
        splitted_themes["eu_data_themes"] = themes
        return splitted_themes
    else:
        for theme in themes:
            if remove_scheme_and_trailing_slash(theme.id) in los:
                splitted_themes["los"].append(theme)
            else:
                splitted_themes["eu_data_themes"].append(theme)
        return splitted_themes


def extend_los_themes(
    themes: List[EuDataTheme], los: Optional[Dict[str, LosNode]]
) -> Optional[List[LosNode]]:
    extended = []
    if los is not None:
        for theme in themes:
            extended.append(los[remove_scheme_and_trailing_slash(str(theme.id))])
    return extended if len(extended) > 0 else None


def extend_eu_data_themes(
    themes: List[EuDataTheme], eu_data_themes: Optional[Dict[str, EuDataTheme]]
) -> Optional[List[EuDataTheme]]:
    extended = []
    if eu_data_themes is not None:
        for theme in themes:
            eu_theme = (
                eu_data_themes.get(remove_scheme_and_trailing_slash(theme.id))
                if theme.id is not None
                else None
            )
            if eu_theme is None:
                extended.append(theme)
            else:
                extended.append(eu_theme)
    return extended if len(extended) > 0 else None


def extend_skos_code(
    skos_code: Optional[SkosCode], references: Optional[Dict[str, SkosCode]]
) -> Optional[SkosCode]:
    ref_code = None
    if references is not None:
        uri = skos_code.uri if skos_code is not None else None
        if uri is not None:
            ref_code = (
                references.get(remove_scheme_and_trailing_slash(uri))
                if references is not None
                else None
            )
            if ref_code is not None:
                return ref_code

    return ref_code


def extend_skos_code_list(
    skos_codes: Optional[List[SkosCode]], references: Optional[Dict[str, SkosCode]]
) -> Optional[List[SkosCode]]:
    if skos_codes is None or references is None:
        return skos_codes
    else:
        extended_codes = []
        for code in skos_codes:
            ref_code = (
                references.get(remove_scheme_and_trailing_slash(code.uri))
                if code.uri is not None
                else None
            )
            if ref_code is not None:
                extended_codes.append(ref_code)
            else:
                extended_codes.append(code)
        return extended_codes


def remove_scheme_and_trailing_slash(uri: Optional[str]) -> str:
    if uri is None:
        return ""

    uri = uri.replace("http://", "").replace("https://", "")

    if uri.endswith("/"):
        return uri[:-1]
    else:
        return uri



from typing import List, Optional

from rdflib import Graph, URIRef
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import Participation
from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    object_value,
    resource_list,
    value_translations,
)
from .agent import extract_agents_for_participation
from .skos_concept import extract_skos_concept


def extract_participations(
    graph: Graph, subject: URIRef
) -> Optional[List[Participation]]:
    values = []
    for resource in resource_list(graph, subject, cv_uri("hasParticipation")):
        resource_uri = resource.toPython() if isinstance(resource, URIRef) else None

        values.append(
            Participation(
                uri=resource_uri,
                identifier=object_value(graph, resource, DCTERMS.identifier),
                description=value_translations(graph, resource, DCTERMS.description),
                role=extract_skos_concept(graph, resource, cv_uri("role")),
                agents=extract_agents_for_participation(graph, resource_uri),
            )
        )

    return values if len(values) > 0 else None



from typing import List, Optional

from rdflib import Graph, URIRef
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import PublicService
from fdk_rdf_parser.rdf_utils import (
    cv_uri,
    object_value,
    resource_list,
    value_list,
    value_translations,
)
from .channel import extract_channels
from .cost import extract_costs
from .criterion_requirement import extract_criterion_requirements
from .dcat_resource import extract_key_words, extract_skos_code_list
from .evidence import extract_evidences
from .harvest_meta_data import extract_meta_data
from .legal_resource import extract_legal_resources
from .output import extract_outputs
from .participation import extract_participations
from .publisher import extract_list_of_publishers
from .rule import extract_rules
from .schema_contact_point import extract_schema_contact_points
from .skos_concept import extract_skos_concept


def extract_public_services(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[List[PublicService]]:
    values = []
    for resource in resource_list(graph, subject, predicate):
        resource_uri = resource.toPython() if isinstance(resource, URIRef) else None
        values.append(
            PublicService(
                uri=resource_uri,
                identifier=object_value(graph, resource, DCTERMS.identifier),
                id=object_value(graph, resource, DCTERMS.identifier),
                title=value_translations(graph, resource, DCTERMS.title),
            )
        )

    return values if len(values) > 0 else None


def parse_public_service(
    public_services_graph: Graph, catalog_record_uri: URIRef, public_service_uri: URIRef
) -> PublicService:

    public_service = PublicService(
        id=object_value(public_services_graph, catalog_record_uri, DCTERMS.identifier),
        uri=public_service_uri.toPython(),
        identifier=object_value(
            public_services_graph, public_service_uri, DCTERMS.identifier
        ),
        title=value_translations(
            public_services_graph, public_service_uri, DCTERMS.title
        ),
        description=value_translations(
            public_services_graph, public_service_uri, DCTERMS.description
        ),
        isGroupedBy=value_list(
            public_services_graph, public_service_uri, cv_uri("isGroupedBy")
        ),
        hasCompetentAuthority=extract_list_of_publishers(
            public_services_graph, public_service_uri, cv_uri("hasCompetentAuthority")
        ),
        harvest=extract_meta_data(public_services_graph, catalog_record_uri),
        keyword=extract_key_words(public_services_graph, public_service_uri),
        sector=extract_skos_concept(
            public_services_graph, public_service_uri, cv_uri("sector")
        ),
        isClassifiedBy=extract_skos_concept(
            public_services_graph, public_service_uri, cv_uri("isClassifiedBy")
        ),
        isDescribedAt=extract_skos_concept(
            public_services_graph, public_service_uri, cv_uri("isDescribedAt")
        ),
        language=extract_skos_code_list(
            public_services_graph, public_service_uri, DCTERMS.language
        ),
        hasCriterion=extract_criterion_requirements(
            public_services_graph, public_service_uri
        ),
        hasParticipation=extract_participations(
            public_services_graph, public_service_uri
        ),
        hasInput=extract_evidences(public_services_graph, public_service_uri),
        produces=extract_outputs(public_services_graph, public_service_uri),
        requires=extract_public_services(
            public_services_graph, public_service_uri, DCTERMS.requires
        ),
        hasContactPoint=extract_schema_contact_points(
            public_services_graph, public_service_uri
        ),
        follows=extract_rules(public_services_graph, public_service_uri),
        hasLegalResource=extract_legal_resources(
            public_services_graph, public_service_uri
        ),
        hasChannel=extract_channels(
            public_services_graph, public_service_uri, cv_uri("hasChannel")
        ),
        processingTime=object_value(
            public_services_graph, public_service_uri, cv_uri("processingTime")
        ),
        hasCost=extract_costs(public_services_graph, public_service_uri),
        relation=extract_public_services(
            public_services_graph, public_service_uri, DCTERMS.relation
        ),
        spatial=value_list(public_services_graph, public_service_uri, DCTERMS.spatial),
    )
    return public_service



from rdflib import Graph, URIRef
from rdflib.namespace import DCTERMS, RDF, XSD

from fdk_rdf_parser.classes.model_property import ModelProperty
from fdk_rdf_parser.rdf_utils import (
    model_dcat_ap_no_uri,
    object_value,
    resource_list,
    uri_or_identifier,
    uri_or_identifier_list,
    value_set,
    value_translations,
)


def parse_model_property(graph: Graph, prop_ref: URIRef) -> ModelProperty:
    prop_uri = None
    if isinstance(prop_ref, URIRef):
        prop_uri = prop_ref.toPython()

    has_type_refs = resource_list(graph, prop_ref, model_dcat_ap_no_uri("hasType"))
    has_some_refs = resource_list(graph, prop_ref, model_dcat_ap_no_uri("hasSome"))

    symmetry_ref = graph.value(prop_ref, model_dcat_ap_no_uri("formsSymmetryWith"))
    data_type_ref = graph.value(prop_ref, model_dcat_ap_no_uri("hasDataType"))
    simple_type_ref = graph.value(prop_ref, model_dcat_ap_no_uri("hasSimpleType"))
    object_type_ref = graph.value(prop_ref, model_dcat_ap_no_uri("hasObjectType"))
    value_from_ref = graph.value(prop_ref, model_dcat_ap_no_uri("hasValueFrom"))
    abstraction_ref = graph.value(prop_ref, model_dcat_ap_no_uri("isAbstractionOf"))
    refers_to_ref = graph.value(prop_ref, model_dcat_ap_no_uri("refersTo"))
    member_ref = graph.value(prop_ref, model_dcat_ap_no_uri("hasMember"))
    contains_ref = graph.value(prop_ref, model_dcat_ap_no_uri("contains"))
    supplier_ref = graph.value(prop_ref, model_dcat_ap_no_uri("hasSupplier"))
    general_concept_ref = graph.value(
        prop_ref, model_dcat_ap_no_uri("hasGeneralConcept")
    )

    return ModelProperty(
        uri=prop_uri,
        identifier=object_value(graph, prop_ref, DCTERMS.identifier),
        propertyTypes=value_set(graph, prop_ref, RDF.type),
        title=value_translations(graph, prop_ref, DCTERMS.title),
        description=value_translations(graph, prop_ref, DCTERMS.description),
        subject=object_value(graph, prop_ref, DCTERMS.subject),
        minOccurs=object_value(graph, prop_ref, XSD.minOccurs),
        maxOccurs=object_value(graph, prop_ref, XSD.maxOccurs),
        relationPropertyLabel=value_translations(
            graph, prop_ref, model_dcat_ap_no_uri("relationPropertyLabel")
        ),
        sequenceNumber=object_value(
            graph, prop_ref, model_dcat_ap_no_uri("sequenceNumber")
        ),
        belongsToModule=object_value(
            graph, prop_ref, model_dcat_ap_no_uri("belongsToModule")
        ),
        hasType=uri_or_identifier_list(graph, has_type_refs),
        formsSymmetryWith=uri_or_identifier(graph, symmetry_ref),
        hasDataType=uri_or_identifier(graph, data_type_ref),
        hasSimpleType=uri_or_identifier(graph, simple_type_ref),
        hasObjectType=uri_or_identifier(graph, object_type_ref),
        hasValueFrom=uri_or_identifier(graph, value_from_ref),
        isAbstractionOf=uri_or_identifier(graph, abstraction_ref),
        refersTo=uri_or_identifier(graph, refers_to_ref),
        hasSome=uri_or_identifier_list(graph, has_some_refs),
        hasMember=uri_or_identifier(graph, member_ref),
        contains=uri_or_identifier(graph, contains_ref),
        hasSupplier=uri_or_identifier(graph, supplier_ref),
        hasGeneralConcept=uri_or_identifier(graph, general_concept_ref),
        notification=value_translations(
            graph, prop_ref, model_dcat_ap_no_uri("notification")
        ),
    )



from fdk_rdf_parser.classes import PublicService
from .reference_data import PublicServiceReferenceData
from .utils import extend_skos_code_list


def extend_public_service_with_reference_data(
    public_service: PublicService, ref_data: PublicServiceReferenceData
) -> PublicService:
    public_service.language = extend_skos_code_list(
        public_service.language, ref_data.linguisticsystem
    )

    return public_service



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Set


@dataclass
class ModelCodeElement:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    prefLabel: Optional[Dict[str, str]] = None
    inScheme: Optional[Set[str]] = None
    subject: Optional[str] = None
    notation: Optional[str] = None
    topConceptOf: Optional[Set[str]] = None
    definition: Optional[Set[str]] = None
    example: Optional[Set[str]] = None
    exclusionNote: Optional[Dict[str, str]] = None
    previousElement: Optional[Set[str]] = None
    hiddenLabel: Optional[Dict[str, str]] = None
    inclusionNote: Optional[Dict[str, str]] = None
    note: Optional[Dict[str, str]] = None
    nextElement: Optional[Set[str]] = None
    scopeNote: Optional[Dict[str, str]] = None
    altLabel: Optional[Dict[str, str]] = None

    def __lt__(self: Any, other: Any) -> bool:
        self_identifier = self.identifier if self.identifier else ""
        other_identifier = other.identifier if other.identifier else ""
        return self_identifier < other_identifier


@dataclass
class ModelElement:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    description: Optional[Dict[str, str]] = None
    subject: Optional[str] = None
    hasProperty: Optional[List[str]] = None
    belongsToModule: Optional[str] = None
    elementTypes: Optional[Set[str]] = None
    codeListReference: Optional[str] = None
    codes: Optional[List[ModelCodeElement]] = None
    typeDefinitionReference: Optional[str] = None
    fractionDigits: Optional[int] = None
    length: Optional[int] = None
    maxInclusive: Optional[float] = None
    maxLength: Optional[int] = None
    minInclusive: Optional[float] = None
    minLength: Optional[int] = None
    pattern: Optional[str] = None
    totalDigits: Optional[int] = None



from typing import List, Optional

from rdflib import Graph, Literal, URIRef
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import ConformsTo
from fdk_rdf_parser.rdf_utils import (
    object_value,
    resource_list,
    value_translations,
)


def extract_conforms_to(graph: Graph, subject: URIRef) -> Optional[List[ConformsTo]]:
    values = []
    for conforms_to in resource_list(graph, subject, DCTERMS.conformsTo):
        conforms_to_uri = None
        if isinstance(conforms_to, URIRef) or isinstance(conforms_to, Literal):
            conforms_to_uri = conforms_to.toPython()

        source_value = object_value(graph, conforms_to, DCTERMS.source)

        values.append(
            ConformsTo(
                uri=source_value if source_value is not None else conforms_to_uri,
                prefLabel=value_translations(graph, conforms_to, DCTERMS.title),
            )
        )

    return values if len(values) > 0 else None



from datetime import date, datetime
import numbers
from typing import Any, Dict, List, Optional, Set

from isodate import date_isoformat, datetime_isoformat
from rdflib import BNode, Graph, URIRef
from rdflib.namespace import DCTERMS, FOAF, RDF


def is_type(t: URIRef, graph: Graph, topic: URIRef) -> bool:
    for type_uri_ref in resource_list(graph, topic, RDF.type):
        if type_uri_ref == t:
            return True

    return False


def is_uri_in_list(
    reference_uri: str, graph: Graph, subject: URIRef, predicate: URIRef
) -> bool:
    return reference_uri in map(
        lambda resource: resource.toPython(), resource_list(graph, subject, predicate)
    )


def object_value(graph: Graph, subject: URIRef, predicate: URIRef) -> Optional[Any]:
    value = graph.value(subject, predicate)
    return value.toPython() if value and not isinstance(value, BNode) else None


def object_number_value(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[Any]:
    value = graph.value(subject, predicate)
    return (
        value.toPython()
        if isinstance(float(value), numbers.Number) and not isinstance(value, BNode)
        else None
    )


def value_list(graph: Graph, subject: URIRef, predicate: URIRef) -> Optional[List[Any]]:
    values = []
    for obj in graph.objects(subject, predicate):
        if not isinstance(obj, BNode):
            values.append(obj.toPython())
    values.sort()
    return values if len(values) > 0 else None


def value_set(graph: Graph, subject: URIRef, predicate: URIRef) -> Optional[Set[Any]]:
    values = set()
    for obj in graph.objects(subject, predicate):
        if not isinstance(obj, BNode):
            values.add(obj.toPython())
    return values if len(values) > 0 else None


def value_translations(
    graph: Graph, subject: URIRef, predicate: URIRef
) -> Optional[Dict[str, str]]:
    values = {}
    for obj in graph.objects(subject, predicate):
        if obj.language:
            values[obj.language] = obj.toPython()
        else:
            values["nb"] = obj.toPython()
    return values if len(values) > 0 else None


def resource_list(graph: Graph, subject: URIRef, predicate: URIRef) -> List[Any]:
    values = []
    for obj in graph.objects(subject, predicate):
        values.append(obj)
    values.sort()
    return values


def uri_or_identifier_list(graph: Graph, subjects: List[URIRef]) -> Optional[List[str]]:
    values = []
    for subject_ref in subjects:
        value = uri_or_identifier(graph, subject_ref)
        if value:
            values.append(value)
    values.sort()
    return values if len(values) > 0 else None


def uri_or_identifier(graph: Graph, subject: URIRef) -> Optional[str]:
    return (
        subject.toPython()
        if isinstance(subject, URIRef)
        else object_value(graph, subject, DCTERMS.identifier)
    )


def date_value(graph: Graph, subject: URIRef, predicate: URIRef) -> Optional[str]:
    value = graph.value(subject, predicate)
    if value:
        date_object = value.toPython()
        if isinstance(date_object, datetime):
            return datetime_isoformat(date_object)
        elif isinstance(date_object, date):
            return date_isoformat(date_object)
        else:
            return None
    else:
        return None


def date_list(graph: Graph, subject: URIRef, predicate: URIRef) -> Optional[List[str]]:
    values = []
    for obj in graph.objects(subject, predicate):
        date_object = obj.toPython()
        if isinstance(date_object, datetime):
            values.append(datetime_isoformat(date_object))
        elif isinstance(date_object, date):
            values.append(date_isoformat(date_object))
    values.sort()
    return values if len(values) > 0 else None


def catalog_ref(graph: Graph, subject: URIRef) -> URIRef:
    catalog_meta_data_ref = graph.value(subject, DCTERMS.isPartOf)
    return graph.value(catalog_meta_data_ref, FOAF.primaryTopic)


linguistic_system_keywords: Dict[str, str] = {
    "http://publications.europa.eu/resource/authority/language/NOR": "no",
    "http://publications.europa.eu/resource/authority/language/NOB": "nb",
    "http://publications.europa.eu/resource/authority/language/NNO": "nn",
    "http://publications.europa.eu/resource/authority/language/ENG": "en",
}



from typing import List, Optional

from rdflib import Graph, URIRef
from rdflib.namespace import DCTERMS, FOAF

from fdk_rdf_parser.classes import Publisher
from fdk_rdf_parser.rdf_utils import (
    object_value,
    value_list,
    value_translations,
)


def extract_publisher(
    graph: Graph, subject: URIRef, catalog_subject: URIRef
) -> Optional[Publisher]:
    publisher = graph.value(subject, DCTERMS.publisher)
    publisher = (
        publisher if publisher else graph.value(catalog_subject, DCTERMS.publisher)
    )

    if publisher:
        return Publisher(
            uri=publisher.toPython() if isinstance(publisher, URIRef) else None,
            id=object_value(graph, publisher, DCTERMS.identifier),
            prefLabel=value_translations(graph, publisher, FOAF.name),
        )
    else:
        return None


def extract_list_of_publishers(
    public_services_graph: Graph, public_service_uri: URIRef, predicate: URIRef
) -> Optional[List[Publisher]]:
    publishers_list = value_list(public_services_graph, public_service_uri, predicate)
    if publishers_list is not None and len(publishers_list) > 0:
        return list(
            map(
                lambda publisher_uri: Publisher(uri=publisher_uri),
                publishers_list,
            )
        )
    else:
        return None



import os
import re
from typing import Optional

base_org_url = os.getenv(
    "ORGANIZATION_CATALOGUE_BASE_URI",
    "https://organization-catalogue.staging.fellesdatakatalog.digdir.no",
)


def organization_url(orgnr: Optional[str]) -> str:
    return (
        f"{base_org_url}/organizations/{orgnr.strip().replace(' ', '')}"
        if orgnr is not None
        else f"{base_org_url}/organizations"
    )


def org_path_url(org: str) -> str:
    return f"{base_org_url}/organizations/orgpath/{org}"


def organization_number_from_uri(uri: str) -> Optional[str]:
    if "data.brreg.no/" in uri or "fellesdatakatalog.digdir.no/organizations/" in uri:
        matches = re.findall("[0-9]{9}", uri)
        return matches[0] if len(matches) == 1 else None
    return None



from dataclasses import dataclass
from typing import Dict, Optional


@dataclass
class LegalResource:
    uri: Optional[str] = None
    description: Optional[Dict[str, str]] = None
    url: Optional[str] = None



from dataclasses import dataclass
from typing import Dict, List, Optional

from .skos_code import SkosCode


@dataclass
class Rule:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    description: Optional[Dict[str, str]] = None
    language: Optional[List[SkosCode]] = None
    name: Optional[Dict[str, str]] = None



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Set

from .catalog import Catalog
from .dcat_resource import PartialDcatResource
from .dct_standard import DctStandard
from .format import Format
from .harvest_meta_data import HarvestMetaData
from .model_element import ModelElement
from .model_property import ModelProperty
from .skos_code import SkosCode
from .temporal import Temporal
from .theme import LosNode


@dataclass
class InformationModel(PartialDcatResource):

    id: Optional[str] = None
    harvest: Optional[HarvestMetaData] = None
    catalog: Optional[Catalog] = None
    conformsTo: Optional[List[DctStandard]] = None
    license: Optional[List[SkosCode]] = None
    informationModelIdentifier: Optional[str] = None
    spatial: Optional[List[SkosCode]] = None
    isPartOf: Optional[str] = None
    hasPart: Optional[str] = None
    isReplacedBy: Optional[str] = None
    isProfileOf: Optional[List[DctStandard]] = None
    replaces: Optional[str] = None
    temporal: Optional[List[Temporal]] = None
    hasFormat: Optional[List[Format]] = None
    homepage: Optional[str] = None
    status: Optional[str] = None
    versionInfo: Optional[str] = None
    versionNotes: Optional[Dict[str, str]] = None
    subjects: Optional[Set[str]] = None
    containsSubjects: Optional[Set[str]] = None
    containsModelElements: Optional[List[str]] = None
    modelElements: Optional[Dict[str, ModelElement]] = None
    modelProperties: Optional[Dict[str, ModelProperty]] = None
    losTheme: Optional[List[LosNode]] = None
    type: str = "informationmodels"

    def add_values_from_dcat_resource(self: Any, values: PartialDcatResource) -> Any:
        self.identifier = values.identifier
        self.publisher = values.publisher
        self.title = values.title
        self.description = values.description
        self.descriptionFormatted = values.descriptionFormatted
        self.uri = values.uri
        self.theme = values.theme
        self.keyword = values.keyword
        self.contactPoint = values.contactPoint
        self.dctType = values.dctType
        self.issued = values.issued
        self.modified = values.modified
        self.language = values.language

    def add_model_element(self: Any, element: ModelElement) -> Any:
        if self.modelElements is None:
            self.modelElements = {}

        self.add_subject_from_element_or_property(element.subject)

        if element.codes:
            [
                self.add_subject_from_element_or_property(code.subject)
                for code in element.codes
            ]

        element_key = element.uri if element.uri else element.identifier
        self.modelElements[element_key] = element

    def add_model_property(self: Any, prop: ModelProperty) -> Any:
        if self.modelProperties is None:
            self.modelProperties = {}

        self.add_subject_from_element_or_property(prop.subject)

        prop_key = prop.uri if prop.uri else prop.identifier
        self.modelProperties[prop_key] = prop

    def add_subject_from_element_or_property(self: Any, subject: Optional[str]) -> Any:
        if subject:
            if self.containsSubjects is None:
                self.containsSubjects = {subject}
            else:
                self.containsSubjects.add(subject)



from functools import reduce
from typing import Dict, List, Optional

from rdflib import Graph
from rdflib.namespace import FOAF, RDF, SKOS

from .classes import (
    Catalog,
    Concept,
    DataService,
    Dataset,
    Event,
    InformationModel,
    PublicService,
    QualifiedAttribution,
)
from .organizations import get_rdf_org_data, publisher_from_fdk_org_catalog
from .parse_functions import (
    extend_with_associated_broader_types,
    parse_concept,
    parse_data_service,
    parse_dataset,
    parse_event,
    parse_information_model,
    parse_public_service,
)
from .rdf_utils import cpsv_uri, cv_uri, dcat_uri, is_type, model_dcat_ap_no_uri
from .reference_data import (
    extend_data_service_with_reference_data,
    extend_dataset_with_reference_data,
    extend_info_model_with_reference_data,
    extend_public_service_with_reference_data,
    get_data_service_reference_data,
    get_dataset_reference_data,
    get_info_model_reference_data,
    get_public_service_reference_data,
)


def parse_data_services(
    data_service_rdf: str, rdf_format: str = "turtle"
) -> Dict[str, DataService]:
    data_services: Dict[str, DataService] = {}
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format="turtle")
    reference_data = get_data_service_reference_data()

    data_services_graph = Graph().parse(data=data_service_rdf, format=rdf_format)

    for record_uri in data_services_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = data_services_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            dcat_uri("DataService"), data_services_graph, primary_topic_uri
        ):
            data_service = parse_data_service(
                data_services_graph, record_uri, primary_topic_uri
            )

            data_service.publisher = (
                publisher_from_fdk_org_catalog(data_service.publisher, fdk_orgs)
                if data_service.publisher
                else None
            )

            data_service.catalog = extend_catalog_with_orgs_data(
                data_service.catalog, fdk_orgs
            )
            data_service = extend_data_service_with_reference_data(
                data_service, reference_data
            )

            data_services[primary_topic_uri.toPython()] = data_service

    return data_services


def parse_datasets(rdf_data: str, rdf_format: str = "turtle") -> Dict[str, Dataset]:
    datasets_graph = Graph().parse(data=rdf_data, format=rdf_format)
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format="turtle")
    reference_data = get_dataset_reference_data()

    datasets: Dict[str, Dataset] = {}

    for record_uri in datasets_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = datasets_graph.value(record_uri, FOAF.primaryTopic)
        if primary_topic_uri is not None and is_type(
            dcat_uri("Dataset"), datasets_graph, primary_topic_uri
        ):
            partial_dataset = parse_dataset(
                datasets_graph, record_uri, primary_topic_uri
            )

            dataset = Dataset(
                publisher=publisher_from_fdk_org_catalog(
                    partial_dataset.publisher, fdk_orgs
                )
                if partial_dataset.publisher
                else None
            )

            dataset.add_values_from_partial(values=partial_dataset)

            dataset = extend_dataset_with_reference_data(dataset, reference_data)
            dataset = extend_dataset_with_orgs_data(dataset, fdk_orgs)
            dataset.catalog = extend_catalog_with_orgs_data(dataset.catalog, fdk_orgs)

            datasets[primary_topic_uri.toPython()] = dataset

    return datasets


def parse_information_models(
    info_models_rdf: str, rdf_format: str = "turtle"
) -> Dict[str, InformationModel]:
    info_models: Dict[str, InformationModel] = {}
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format="turtle")
    reference_data = get_info_model_reference_data()

    info_models_graph = Graph().parse(data=info_models_rdf, format=rdf_format)

    for record_uri in info_models_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = info_models_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            model_dcat_ap_no_uri("InformationModel"),
            info_models_graph,
            primary_topic_uri,
        ):
            info_model = parse_information_model(
                info_models_graph, record_uri, primary_topic_uri
            )

            info_model.publisher = (
                publisher_from_fdk_org_catalog(info_model.publisher, fdk_orgs)
                if info_model.publisher
                else None
            )

            info_model.catalog = extend_catalog_with_orgs_data(
                info_model.catalog, fdk_orgs
            )
            info_model = extend_info_model_with_reference_data(
                info_model, reference_data
            )

            info_models[primary_topic_uri.toPython()] = info_model

    return info_models


def parse_public_services(
    public_service_rdf: str, event_rdf: str = None, rdf_format: str = "turtle"
) -> Dict[str, PublicService]:
    public_services: Dict[str, PublicService] = {}
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format=rdf_format)
    reference_data = get_public_service_reference_data()

    events: Dict[str, Optional[Event]] = (
        parse_events(event_rdf) if event_rdf is not None else {}
    )

    public_services_graph = Graph().parse(data=public_service_rdf, format=rdf_format)

    for catalog_record_uri in public_services_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = public_services_graph.value(
            catalog_record_uri, FOAF.primaryTopic
        )

        if primary_topic_uri and is_type(
            cpsv_uri("PublicService"),
            public_services_graph,
            primary_topic_uri,
        ):
            public_service = parse_public_service(
                public_services_graph, catalog_record_uri, primary_topic_uri
            )

            if (
                public_service.hasCompetentAuthority is not None
                and len(public_service.hasCompetentAuthority) > 0
            ):
                public_service.hasCompetentAuthority = list(
                    filter(
                        None,
                        map(
                            lambda authority: publisher_from_fdk_org_catalog(
                                authority, fdk_orgs
                            ),
                            public_service.hasCompetentAuthority,
                        ),
                    )
                )

            if public_service.hasCost is not None and len(public_service.hasCost) > 0:
                costs = []
                for cost in public_service.hasCost:
                    if cost.isDefinedBy is not None and len(cost.isDefinedBy) > 0:
                        cost.isDefinedBy = list(
                            filter(
                                None,
                                map(
                                    lambda isDefinedBy: publisher_from_fdk_org_catalog(
                                        isDefinedBy, fdk_orgs
                                    ),
                                    cost.isDefinedBy,
                                ),
                            )
                        )
                    costs.append(cost)
                public_service.hasCost = costs

            if (
                public_service.isGroupedBy is not None
                and len(public_service.isGroupedBy) > 0
            ):
                associated_skos_concepts_uris: List[str] = []
                reduce(
                    lambda output, current_event_uri: extend_with_associated_broader_types(
                        events, current_event_uri, output
                    ),
                    public_service.isGroupedBy,
                    associated_skos_concepts_uris,
                )
                public_service.associatedBroaderTypesByEvents = (
                    associated_skos_concepts_uris
                    if associated_skos_concepts_uris
                    and len(associated_skos_concepts_uris) > 0
                    else None
                )

            public_service = extend_public_service_with_reference_data(
                public_service, reference_data
            )

            public_services[primary_topic_uri.toPython()] = public_service

    return public_services


def parse_events(
    event_rdf: str, rdf_format: str = "turtle"
) -> Dict[str, Optional[Event]]:
    events: Dict[str, Optional[Event]] = {}

    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format=rdf_format)
    graph = Graph().parse(data=event_rdf, format=rdf_format)

    for catalog_record_uri in graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = graph.value(catalog_record_uri, FOAF.primaryTopic)

        if primary_topic_uri and (
            is_type(
                cv_uri("BusinessEvent"),
                graph,
                primary_topic_uri,
            )
            or is_type(
                cv_uri("LifeEvent"),
                graph,
                primary_topic_uri,
            )
        ):
            event = parse_event(graph, catalog_record_uri, primary_topic_uri)

            if (
                event
                and event.hasCompetentAuthority is not None
                and len(event.hasCompetentAuthority) > 0
            ):
                event.hasCompetentAuthority = list(
                    filter(
                        None,
                        map(
                            lambda authority: publisher_from_fdk_org_catalog(
                                authority, fdk_orgs
                            ),
                            event.hasCompetentAuthority,
                        ),
                    )
                )

            events[primary_topic_uri.toPython()] = event

    return events


def parse_concepts(concepts_rdf: str, rdf_format: str = "turtle") -> Dict[str, Concept]:
    concepts: Dict[str, Concept] = {}
    fdk_orgs = Graph().parse(data=get_rdf_org_data(orgnr=None), format="turtle")

    concepts_graph = Graph().parse(data=concepts_rdf, format=rdf_format)

    for record_uri in concepts_graph.subjects(
        predicate=RDF.type, object=dcat_uri("CatalogRecord")
    ):
        primary_topic_uri = concepts_graph.value(record_uri, FOAF.primaryTopic)

        if primary_topic_uri and is_type(
            SKOS.Concept,
            concepts_graph,
            primary_topic_uri,
        ):
            concept = parse_concept(concepts_graph, record_uri, primary_topic_uri)

            concept.publisher = publisher_from_fdk_org_catalog(
                concept.publisher, fdk_orgs
            )
            if concept.collection:
                concept.collection.publisher = publisher_from_fdk_org_catalog(
                    concept.collection.publisher, fdk_orgs
                )

            concepts[primary_topic_uri.toPython()] = concept

    return concepts


def extend_catalog_with_orgs_data(
    catalog: Optional[Catalog], organizations_graph: Graph
) -> Optional[Catalog]:
    if catalog:
        catalog.publisher = publisher_from_fdk_org_catalog(
            catalog.publisher, organizations_graph
        )
    return catalog


def extend_dataset_with_orgs_data(
    dataset: Dataset, organizations_graph: Graph
) -> Dataset:
    if isinstance(dataset.qualifiedAttributions, list):
        dataset.qualifiedAttributions = list(
            map(
                lambda qa: enhance_qualified_attribution_agent(qa, organizations_graph),
                dataset.qualifiedAttributions,
            )
        )

    return dataset


def enhance_qualified_attribution_agent(
    qa: QualifiedAttribution, organizations_graph: Graph
) -> QualifiedAttribution:
    qa.agent = publisher_from_fdk_org_catalog(qa.agent, organizations_graph)

    return qa



import re
from typing import Dict, List, Optional

from rdflib import Graph, URIRef
from rdflib.namespace import DCTERMS

from fdk_rdf_parser.classes import EuDataTheme, PartialDcatResource
from fdk_rdf_parser.rdf_utils import (
    date_value,
    dcat_uri,
    object_value,
    value_list,
    value_set,
    value_translations,
)
from .contactpoint import extract_contact_points
from .publisher import extract_publisher
from .skos_code import extract_skos_code, extract_skos_code_list


def parse_dcat_resource(
    graph: Graph, subject: URIRef, catalog_subject: URIRef
) -> PartialDcatResource:
    formatted_description = value_translations(graph, subject, DCTERMS.description)
    return PartialDcatResource(
        identifier=value_set(graph, subject, DCTERMS.identifier),
        publisher=extract_publisher(graph, subject, catalog_subject),
        title=value_translations(graph, subject, DCTERMS.title),
        description=description_html_cleaner(formatted_description)
        if formatted_description
        else None,
        descriptionFormatted=formatted_description,
        uri=subject.toPython(),
        accessRights=extract_skos_code(graph, subject, DCTERMS.accessRights),
        theme=extract_themes(graph, subject),
        keyword=extract_key_words(graph, subject),
        contactPoint=extract_contact_points(graph, subject),
        dctType=object_value(graph, subject, DCTERMS.type),
        issued=date_value(graph, subject, DCTERMS.issued),
        modified=date_value(graph, subject, DCTERMS.modified),
        landingPage=value_set(graph, subject, dcat_uri("landingPage")),
        language=extract_skos_code_list(graph, subject, DCTERMS.language),
    )


def extract_themes(graph: Graph, subject: URIRef) -> Optional[List[EuDataTheme]]:
    themes = value_list(graph, subject, dcat_uri("theme"))
    if themes is not None and len(themes) > 0:
        return list(map(lambda theme_uri: EuDataTheme(id=theme_uri), themes))
    else:
        return None


def extract_key_words(graph: Graph, subject: URIRef) -> Optional[List[Dict[str, str]]]:
    values = []
    for keyword in graph.objects(subject, dcat_uri("keyword")):
        translation = {}
        if keyword.language:
            translation[keyword.language] = keyword.toPython()
        else:
            translation["nb"] = keyword.toPython()
        values.append(translation)
    return values if len(values) > 0 else None


def description_html_cleaner(formatted: Dict[str, str]) -> Optional[Dict[str, str]]:
    cleaner_regex = re.compile("<.*?>|&([a-z0-9]+|#[0-9]{1,6}|#x[0-9a-f]{1,6});")
    description = {}
    for language in formatted:
        description[language] = re.sub(cleaner_regex, "", formatted[language])

    return description if len(description) > 0 else None



from typing import Optional

from rdflib import Graph, URIRef
from rdflib.namespace import FOAF, SKOS

from fdk_rdf_parser.classes import Publisher
from fdk_rdf_parser.rdf_utils import br_uri, object_value, rov_uri, value_translations
from .organizations_client import get_org_path, get_rdf_org_data
from .utils import organization_number_from_uri, organization_url


def publisher_from_fdk_org_catalog(
    publisher: Optional[Publisher], orgs_graph: Graph
) -> Optional[Publisher]:
    if publisher:
        if publisher.id:
            return get_publisher_data_from_organization_catalogue(publisher, orgs_graph)
        elif publisher.uri:
            publisher.id = organization_number_from_uri(publisher.uri)

            if publisher.id:
                return get_publisher_data_from_organization_catalogue(
                    publisher, orgs_graph
                )
            else:
                return add_org_path(publisher)
        else:
            return add_org_path(publisher)
    else:
        return None


def get_publisher_data_from_organization_catalogue(
    publisher: Publisher, orgs_graph: Graph
) -> Publisher:
    org_ref = URIRef(organization_url(publisher.id))

    if (org_ref, None, None) in orgs_graph:
        return parse_publisher(orgs_graph, org_ref, publisher)
    else:
        fdk_org = get_rdf_org_data(orgnr=publisher.id)

        if fdk_org is not None:
            org_graph = Graph().parse(data=fdk_org, format="turtle")
            return parse_publisher(org_graph, org_ref, publisher)
        else:
            return add_org_path(publisher)


def parse_publisher(graph: Graph, org_ref: URIRef, publisher: Publisher) -> Publisher:
    reg_data_ref = graph.value(org_ref, rov_uri("registration"))

    return Publisher(
        id=object_value(graph, reg_data_ref, SKOS.notation),
        uri=org_ref.toPython(),
        name=object_value(graph, org_ref, rov_uri("legalName")),
        orgPath=object_value(graph, org_ref, br_uri("orgPath")),
        prefLabel=publisher.prefLabel
        if publisher.prefLabel
        else value_translations(graph, org_ref, FOAF.name),
        organisasjonsform=object_value(graph, org_ref, rov_uri("orgType")),
    )


def add_org_path(publisher: Publisher) -> Publisher:
    org_path = None
    if publisher.id:
        org_path = get_org_path(publisher.id.strip().replace(" ", ""))
    elif publisher.prefLabel:
        if publisher.prefLabel.get("nb"):
            org_path = get_org_path(publisher.prefLabel["nb"])
        elif publisher.prefLabel.get("nn"):
            org_path = get_org_path(publisher.prefLabel["nn"])
        elif publisher.prefLabel.get("en"):
            org_path = get_org_path(publisher.prefLabel["en"])

    publisher.orgPath = org_path
    return publisher



from dataclasses import dataclass
from typing import Dict, List, Optional


@dataclass
class Agent:
    uri: Optional[str] = None
    identifier: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    name: Optional[str] = None
    playsRole: Optional[List[str]] = None



from dataclasses import dataclass
from typing import Dict, List, Optional


@dataclass
class DctStandard:
    uri: Optional[str] = None
    title: Optional[Dict[str, str]] = None
    seeAlso: Optional[List[str]] = None
    versionInfo: Optional[str] = None
